@@ -1,9 +1,34 @@
+/* $EPIC: files.c,v 1.23 2008/01/15 13:04:54 jnelson Exp $ */
 /*
  * files.c -- allows you to read/write files. Wow.
  *
- * Written by Jeremy Nelson
- * Copyright 1995 Jeremy Nelson
- * See the COPYRIGHT file for more information
+ * Copyright  1995, 2003 EPIC Software Labs
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notices, the above paragraph (the one permitting redistribution),
+ *    this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The names of the author(s) may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "irc.h"
@@ -43,20 +68,20 @@ typedef struct FILE___ File;
 
 static File *FtopEntry = (File *) 0;
 
-File *new_file (void)
+static File *new_file (void)
 {
 	File *tmp = FtopEntry;
-	File *tmpfile = (File *)new_malloc(sizeof(File));
+	File *tmp_file = (File *)new_malloc(sizeof(File));
 
 	if (FtopEntry == (File *) 0)
-		FtopEntry = tmpfile;
+		FtopEntry = tmp_file;
 	else
 	{
 		while (tmp->next)
 			tmp = tmp->next;
-		tmp->next = tmpfile;
+		tmp->next = tmp_file;
 	}
-	return tmpfile;
+	return tmp_file;
 }
 
 static void remove_file (File *file)
@@ -77,12 +102,11 @@ static void remove_file (File *file)
 }
 
 
-int open_file_for_read (char *filename)
+int open_file_for_read (const char *filename)
 {
 	char *dummy_filename = (char *) 0;
 	FILE *file;
 
-	/* XXXX - this looks like a memory leak */
 	malloc_strcpy(&dummy_filename, filename);
 	file = uzfopen(&dummy_filename, ".", 1);
 	new_free(&dummy_filename);
@@ -98,17 +122,15 @@ int open_file_for_read (char *filename)
 		return -1;
 }
 
-int open_file_for_write (char *filename)
+int open_file_for_write (const char *filename, const char *mode)
 {
-	/* patch by Scott H Kilau so expand_twiddle works */
-	char *expand = NULL;
+	Filename expand;
 	FILE *file;
 
-	if (!(expand = expand_twiddle(filename)))
-		malloc_strcpy(&expand, filename);
-	file = fopen(expand, "a");
-	new_free(&expand);
-	if (file)
+	if (normalize_filename(filename, expand))
+		strlcpy(expand, filename, sizeof(expand));
+
+	if ((file = fopen(expand, mode)))
 	{
 		File *nfs = new_file();
 		nfs->file = file;
@@ -119,6 +141,30 @@ int open_file_for_write (char *filename)
 		return -1;
 }
 
+int* open_exec_for_in_out_err (const char *filename, char * const *args)
+{
+	Filename expand;
+	FILE **files;
+static	int ret[3];
+
+	if (normalize_filename(filename, expand))
+		strlcpy(expand, filename, sizeof(expand));
+
+	if ((files = open_exec(filename, args)))
+	{
+		int foo;
+		for (foo = 0; foo < 3; foo++) {
+			File *nfs = new_file();
+			nfs->file = files[foo];
+			nfs->next = (File *) 0;
+			ret[foo] = fileno(files[foo]);
+		}
+		return ret;
+	}
+	else 
+		return NULL;
+}
+
 static File *lookup_file (int fd)
 {
 	File *ptr = FtopEntry;
@@ -141,8 +187,10 @@ static File *lookup_logfile (int fd)
 
 	if (fd == -1)
 		x = irclog_fp;
-	if ((w = get_window_by_refnum(fd)))
+	else if ((w = get_window_by_refnum(fd)))
 		x = w->log_fp;
+	else
+		return NULL;
 
 	retval.file = x;		/* XXX Should be a file */
 	retval.next = NULL;
@@ -182,6 +230,7 @@ int file_writeb (int window, int fd, char *stuff)
 {
 	File 	*ptr;
 	int	retval;
+	size_t	len = strlen(stuff);
 
 	if (window == 1)
 		ptr = lookup_logfile(fd);
@@ -191,7 +240,10 @@ int file_writeb (int window, int fd, char *stuff)
 	if (!ptr || !ptr->file)
 		return -1;
 
-	retval = fwrite(stuff, 1, strlen(stuff), ptr->file);
+	stuff = dequote_it(stuff, &len);
+	retval = fwrite(stuff, 1, len, ptr->file);
+	new_free(&stuff);
+
 	if ((fflush(ptr->file)) == EOF)
 		return -1;
 	return retval;
@@ -201,7 +253,7 @@ char *file_read (int fd)
 {
 	File *ptr = lookup_file(fd);
 	if (!ptr)
-		return m_strdup(empty_string);
+		return malloc_strdup(empty_string);
 	else
 	{
 		char	*ret = NULL;
@@ -209,6 +261,8 @@ char *file_read (int fd)
 		size_t	len = 0;
 		size_t	newlen = 0;
 
+		clearerr(ptr->file);
+
 		for (;;)
 		{
 		    newlen += 4096;
@@ -235,13 +289,16 @@ char *file_readb (int fd, int numb)
 {
 	File *ptr = lookup_file(fd);
 	if (!ptr)
-		return m_strdup(empty_string);
+		return malloc_strdup(empty_string);
 	else
 	{
 		char *blah = (char *)new_malloc(numb+1);
-		fread(blah, 1, numb, ptr->file);
-		blah[numb] = 0;
-		return blah;
+		char *bleh = NULL;
+		clearerr(ptr->file);
+		numb = fread(blah, 1, numb, ptr->file);
+		bleh = enquote_it(blah, numb);
+		new_free(&blah);
+		return bleh;
 	}
 }
 
@@ -291,6 +348,15 @@ int	file_seek (int fd, long offset, const char *whence)
 		return -1;
 }
 
+int	file_tell (int fd)
+{
+	File *ptr = lookup_file (fd);
+	if (!ptr)
+		return -1;
+	else
+		return ftell(ptr->file);
+}
+
 int	file_skip (int fd, int lines)
 {
 	int line = 0;
