@@ -1,18 +1,54 @@
+/* $EPIC: reg.c,v 1.9 2004/11/10 03:20:35 jnelson Exp $ */
 /*
+ * reg.c - "glob"-like wildcard pattern matching (not regexes)
+ *
+ * Copyright (c) 1992 Troy Rollo.
+ * Copyright  1997, 2002 EPIC Software Labs.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notices, the above paragraph (the one permitting redistribution),
+ *    this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The names of the author(s) may not be used to endorse or promote
+ *    products derived from this software without specific prior written
+ *    permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
+ * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+ * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
+
+/*
+ * [Original note from Troy]
  * The original was spagetti. I have replaced Michael's code with some of
  * my own which is a thousand times more readable and can also handle '%',
  * which substitutes anything except a space. This should enable people
  * to position things better based on argument. I have also added '?', which
  * substitutes to any single character. And of course it still handles '*'.
  * this should be more efficient than the previous version too.
- *
- * Thus this whole file becomes:
- *
- * Written By Troy Rollo
- * Copyright(c) 1992
- * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
  */
-
+/*
+ * [Note from EPIC Software Labs]
+ * I had seen that phone had written a non-recursive pattern matcher for ircII,
+ * so I decided to see if I could write one too without peeking at his 
+ * implementation.  The matcher I came up with ("new_match") is iterative and 
+ * is bug-for-bug compatable with troy's recursive matcher ("old_match"). 
+ * It's a lot more complicated, but it's also a lot faster.  (jfn)
+ */
 
 #include "irc.h"
 #include "ircaux.h"
@@ -28,7 +64,7 @@ static	int	total_explicit;
  */
 #define	mkupper(c)	(((c) >= 'a' && (c) <= 'z') ? ((c) - 'a' + 'A') : c)
 
-int old_match(const char *pattern, const char *string)
+static int	old_match(const char *pattern, const char *string)
 {
 	char	type = 0;
 
@@ -57,7 +93,7 @@ int old_match(const char *pattern, const char *string)
 		{
 			if (old_match(pattern, string))
 				return 1;
-			else if (type == '*' || *string != ' ')
+			else if (type == '*' || !isspace(*string))
 				string++;
 			else
 				break;
@@ -75,7 +111,7 @@ int old_match(const char *pattern, const char *string)
 	return 0;
 }
 
-int new_match (const unsigned char *pattern, const unsigned char *string)
+static int new_match (const unsigned char *pattern, const unsigned char *string)
 {
 	int		count = 1;
 	int 		asterisk = 0;
@@ -128,14 +164,33 @@ int new_match (const unsigned char *pattern, const unsigned char *string)
 			}
 
 			/*
-			 * XXXX Skip over any backslashes...
+			 * A backslash after a * sits there until we find
+		 	 * a char to match the character after it.  Then we
+			 * release the \, and the next iteration it will 
+			 * pass the following test...
 			 */
 			if (*pattern == '\\')
 			{
+			   if (x_debug & DEBUG_REGEX_DEBUG)
+			      yell("Trying to match [%d] after the backslash "
+                                   "against [%d]", (int)*(pattern + 1), 
+						   (int)*string);
+
+			   if (tolower(*string) == tolower(*(pattern+1)))
+			   {
+			        if (x_debug & DEBUG_REGEX_DEBUG)
+			           yell("It matches!  Releasing the backslash");
+
 				pattern++;
-				/* XXX This test looks totaly pointless. */
-				if (tolower(*string) != tolower(*pattern))
-					continue;
+				asterisk = 0;
+				last_asterisk_point = string;
+				last_asterisk_count = count;
+
+				pattern++;
+				string++;
+			   }
+			   else
+				string++;		/* Not this char */
 			}
 
 			/*
@@ -199,13 +254,41 @@ int new_match (const unsigned char *pattern, const unsigned char *string)
 			}
 
 			/*
-			 * XXXX Skip over any backslashes...
+			 * A backslash after a % sits there until we find
+		 	 * a char to match the character after it.  Then we
+			 * release the \, and the next iteration it will 
+			 * pass the following test...
 			 */
 			if (*pattern == '\\')
 			{
+			   if (x_debug & DEBUG_REGEX_DEBUG)
+			      yell("Trying to match [%d] after the backslash "
+                                   "against [%d]", (int)*(pattern + 1), 
+						   (int)*string);
+
+			   if (tolower(*string) == tolower(*(pattern+1)))
+			   {
+			        if (x_debug & DEBUG_REGEX_DEBUG)
+			           yell("It matches!  Releasing the backslash");
+
 				pattern++;
-				if (tolower(*string) != tolower(*pattern))
-					continue;
+				asterisk = 0;
+				last_asterisk_point = string;
+				last_asterisk_count = count;
+
+				pattern++;
+				string++;
+			   }
+			   else if (isspace(*string))
+			   {
+				if (x_debug & DEBUG_REGEX_DEBUG)
+				{
+				   yell("Found a space trying to match the [%d] after a %%\\, so this doesn't match.", (int)*(pattern + 1));
+				   return 0;
+				}
+			   }
+			   else
+				string++;		/* Not this char */
 			}
 
 			/*
@@ -214,14 +297,14 @@ int new_match (const unsigned char *pattern, const unsigned char *string)
 			 * go back to normal parsing mode, presumably with
 			 * the space after the %.
 			 */
-			if (*string == ' ')
+			else if (isspace(*string))
 			{
 				if (x_debug & DEBUG_REGEX_DEBUG)
 					yell("Found a space");
 				percent = 0;
 				last_percent_point = NULL;
 			}
-   
+
 			/*
 			 * If this is not the char we're looking for, then
 			 * keep looking.
@@ -511,9 +594,9 @@ int wild_match (const char *p, const char *str)
 				int tmpval;
 				char my_buff[BIG_BUFFER_SIZE + 1];
 
-				strlcpy(my_buff, pattern, BIG_BUFFER_SIZE);
-				strlcat(my_buff, arg, BIG_BUFFER_SIZE);
-				strlcat(my_buff, ptr, BIG_BUFFER_SIZE);
+				strlcpy(my_buff, pattern, sizeof my_buff);
+				strlcat(my_buff, arg, sizeof my_buff);
+				strlcat(my_buff, ptr, sizeof my_buff);
 
 				/*
 				 * The total_explicit we return is whatever
@@ -610,10 +693,10 @@ char *	pattern2regex (const char *pattern, int *weight)
 			strextend(retval, '.', 1);
 			break;
 		case '%' :
-			strcat(retval, "[^ \t]*");
+			strlcat(retval, "[^ \t]*", retsize);
 			break;
 		case '*' :
-			strcat(retval, ".*");
+			strlcat(retval, ".*", retsize);
 			break;
 		case '\\' :
 			if (pat[1] != '[')
@@ -652,15 +735,15 @@ char *	pattern2regex (const char *pattern, int *weight)
 			placeholder += 2;
 
 			arg = new_next_arg(placeholder, &placeholder);
-			strcat(retval, "(");
-			strcat(retval, arg);
+			strlcat(retval, "(", retsize);
+			strlcat(retval, arg, retsize);
 
 			while ((arg = new_next_arg(placeholder, &placeholder)))
 			{
-				strcat(retval, "|");
-				strcat(retval, arg);
+				strlcat(retval, "|", retsize);
+				strlcat(retval, arg, retsize);
 			}
-			strcat(retval, ")");
+			strlcat(retval, ")", retsize);
 			break;
 		   }
 		   end:
